/*
*********************************************************************************************************
*                                     MICIRUM BOARD SUPPORT PACKAGE
*
*                             (c) Copyright 2014; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                     BOARD SUPPORT PACKAGE (BSP)
*
*                                       IAR Development Kits
*                                              on the
*
*                                    M451
*
* Filename      : bsp.h
* Version       : V1.00
* Programmer(s) : FF
*                 DC
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                                 MODULE
*
* Note(s) : (1) This header file is protected from multiple pre-processor inclusion through use of the
*               BSP present pre-processor macro definition.
*
*           (2) This file and its dependencies requires IAR v6.20 or later to be compiled.
*
*********************************************************************************************************
*/

#ifndef  BSP_PRESENT
#define  BSP_PRESENT


/*
*********************************************************************************************************
*                                                 EXTERNS
*********************************************************************************************************
*/

#ifdef   BSP_MODULE
#define  BSP_EXT
#else
#define  BSP_EXT  extern
#endif


/*
*********************************************************************************************************
*                                              INCLUDE FILES
*********************************************************************************************************
*/

#include  <stdio.h>
#include  <stdarg.h>

#include  <cpu.h>
#include  <cpu_core.h>

#include  <lib_def.h>
#include  <lib_ascii.h>


#include "M451Series.h"
/*
*********************************************************************************************************
*                                               CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                             PERIPH DEFINES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                               DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                                 MACRO'S
*********************************************************************************************************
*/



#define      BOD_IRQn                      0       /*!< Brown Out detection Interrupt                    */  
#define      IRC_IRQn                      1       /*!< Internal RC Interrupt                            */  
#define      PWRWU_IRQn                    2       /*!< Power Down Wake Up Interrupt                     */  
#define      RAMPE_IRQn                    3       /*!< SRAM parity check failed Interrupt               */  
#define      CKFAIL_IRQn                   4       /*!< Clock failed Interrupt                           */  
#define      RTC_IRQn                      6       /*!< Real Time Clock Interrupt                        */  
#define      TAMPER_IRQn                   7       /*!< Tamper detection Interrupt                       */  
#define      WDT_IRQn                      8       /*!< Watchdog Timer Interrupt                         */  
#define      WWDT_IRQn                     9       /*!< Window Watchdog Timer Interrupt                  */  
#define      EINT0_IRQn                    10       /*!< External Input 0 Interrupt                       */  
#define      EINT1_IRQn                    11       /*!< External Input 1 Interrupt                       */  
#define      EINT2_IRQn                    12       /*!< External Input 2 Interrupt                       */  
#define      EINT3_IRQn                    13       /*!< External Input 3 Interrupt                       */  
#define      EINT4_IRQn                    14       /*!< External Input 4 Interrupt                       */  
#define      EINT5_IRQn                    15       /*!< External Input 5 Interrupt                       */  
#define      GPA_IRQn                      16       /*!< GPIO Port A Interrupt                            */  
#define      GPB_IRQn                      17       /*!< GPIO Port B Interrupt                            */  
#define      GPC_IRQn                      18       /*!< GPIO Port C Interrupt                            */  
#define      GPD_IRQn                      19       /*!< GPIO Port D Interrupt                            */  
#define      GPE_IRQn                      20       /*!< GPIO Port E Interrupt                            */  
#define      GPF_IRQn                      21       /*!< GPIO Port F Interrupt                            */  
#define      SPI0_IRQn                     22       /*!< SPI0 Interrupt                                   */  
#define      SPI1_IRQn                     23       /*!< SPI1 Interrupt                                   */  
#define      BRAKE0_IRQn                   24       /*!< BRAKE0 Interrupt                                 */  
#define      PWM0P0_IRQn                   25       /*!< PWM0P0 Interrupt                                 */  
#define      PWM0P1_IRQn                   26       /*!< PWM0P1 Interrupt                                 */  
#define      PWM0P2_IRQn                   27       /*!< PWM0P2 Interrupt                                 */  
#define      BRAKE1_IRQn                   28       /*!< BRAKE1 Interrupt                                 */  
#define      PWM1P0_IRQn                   29       /*!< PWM1P0 Interrupt                                 */  
#define      PWM1P1_IRQn                   30       /*!< PWM1P1 Interrupt                                 */  
#define      PWM1P2_IRQn                   31       /*!< PWM1P2 Interrupt                                 */  
#define      TMR0_IRQn                     32       /*!< Timer 0 Interrupt                                */  
#define      TMR1_IRQn                     33       /*!< Timer 1 Interrupt                                */  
#define      TMR2_IRQn                     34       /*!< Timer 2 Interrupt                                */  
#define      TMR3_IRQn                     35       /*!< Timer 3 Interrupt                                */  
#define      UART0_IRQn                    36       /*!< UART 0 Interrupt                                 */  
#define      UART1_IRQn                    37       /*!< UART 1 Interrupt                                 */  
#define      I2C0_IRQn                     38       /*!< I2C 0 Interrupt                                  */  
#define      I2C1_IRQn                     39       /*!< I2C 1 Interrupt                                  */  
#define      PDMA_IRQn                     40       /*!< Peripheral DMA Interrupt                         */  
#define      DAC_IRQn                      41       /*!< DAC Interrupt                                    */  
#define      ADC00_IRQn                    42       /*!< ADC0 Source 0 Interrupt                          */  
#define      ADC01_IRQn                    43       /*!< ADC0 Source 1 Interrupt                          */  
#define      ACMP01_IRQn                   44       /*!< Analog Comparator 0 and 1 Interrupt              */  
#define      ADC02_IRQn                    46       /*!< ADC0 Source 2 Interrupt                          */  
#define      ADC03_IRQn                    47       /*!< ADC0 Source 3 Interrupt                          */  
#define      UART2_IRQn                    48       /*!< UART2 Interrupt                                  */  
#define      UART3_IRQn                    49       /*!< UART3 Interrupt                                  */  
#define      SPI2_IRQn                     51       /*!< SPI2 Interrupt                                   */  
#define      USBD_IRQn                     53       /*!< USB device Interrupt                             */  
#define      USBH_IRQn                     54       /*!< USB host Interrupt                               */  
#define      USBOTG_IRQn                   55       /*!< USB OTG Interrupt                                */  
#define      CAN0_IRQn                     56       /*!< CAN0 Interrupt                                   */  
#define      SC0_IRQn                      58       /*!< Smart Card 0 Interrupt                           */  
#define      TK_IRQn                       63        /*!< Touch Key Interrupt                              */  
#endif

/*
*********************************************************************************************************
*                                             PERIPH DEFINES
*********************************************************************************************************
*/

#if 0
                                                                /* -                  AHB1 BUS                        - */
#define  BSP_PERIPH_ID_GPIOA                               0u
#define  BSP_PERIPH_ID_GPIOB                               1u
#define  BSP_PERIPH_ID_GPIOC                               2u
#define  BSP_PERIPH_ID_GPIOD                               3u
#define  BSP_PERIPH_ID_GPIOE                               4u
#define  BSP_PERIPH_ID_GPIOF                               5u
#define  BSP_PERIPH_ID_GPIOG                               6u
#define  BSP_PERIPH_ID_GPIOH                               7u
#define  BSP_PERIPH_ID_GPIOI                               8u
#define  BSP_PERIPH_ID_GPIOJ                               9u
#define  BSP_PERIPH_ID_GPIOK                              10u
#define  BSP_PERIPH_ID_CRC                                12u
#define  BSP_PERIPH_ID_BKPSRAM                            18u
#define  BSP_PERIPH_ID_CCMDATARAM                         20u
#define  BSP_PERIPH_ID_DMA1                               21u
#define  BSP_PERIPH_ID_DMA2                               22u
#define  BSP_PERIPH_ID_DMA2D                              23u
#define  BSP_PERIPH_ID_ETHMAC                             25u
#define  BSP_PERIPH_ID_ETHMACTX                           26u
#define  BSP_PERIPH_ID_ETHMACRX                           27u
#define  BSP_PERIPH_ID_ETHMACPTP                          28u
#define  BSP_PERIPH_ID_OTGHS                              29u
#define  BSP_PERIPH_ID_OTGHSULPI                          30u
                                                                /* -                  AHB2 BUS                        - */
#define  BSP_PERIPH_ID_DCMI                               32u
#define  BSP_PERIPH_ID_CRYP                               36u
#define  BSP_PERIPH_ID_HASH                               37u
#define  BSP_PERIPH_ID_RNG                                38u
#define  BSP_PERIPH_ID_OTGFS                              39u
                                                                /* -                  AHB3 BUS                        - */
#define  BSP_PERIPH_ID_FSMC                               64u
                                                                /* -                  APB1 BUS                        - */
#define  BSP_PERIPH_ID_TIM2                               96u
#define  BSP_PERIPH_ID_TIM3                               97u
#define  BSP_PERIPH_ID_TIM4                               98u
#define  BSP_PERIPH_ID_TIM5                               99u
#define  BSP_PERIPH_ID_TIM6                              100u
#define  BSP_PERIPH_ID_TIM7                              101u
#define  BSP_PERIPH_ID_TIM12                             102u
#define  BSP_PERIPH_ID_TIM13                             103u
#define  BSP_PERIPH_ID_TIM14                             104u
#define  BSP_PERIPH_ID_WWDG                              107u
#define  BSP_PERIPH_ID_SPI2                              110u
#define  BSP_PERIPH_ID_SPI3                              111u
#define  BSP_PERIPH_ID_USART2                            113u
#define  BSP_PERIPH_ID_USART3                            114u
#define  BSP_PERIPH_ID_UART4                             115u
#define  BSP_PERIPH_ID_UART5                             116u
#define  BSP_PERIPH_ID_I2C1                              117u
#define  BSP_PERIPH_ID_I2C2                              118u
#define  BSP_PERIPH_ID_I2C3                              119u
#define  BSP_PERIPH_ID_CAN1                              121u
#define  BSP_PERIPH_ID_CAN2                              122u
#define  BSP_PERIPH_ID_PWR                               124u
#define  BSP_PERIPH_ID_DAC                               125u
#define  BSP_PERIPH_ID_UART7                             126u
#define  BSP_PERIPH_ID_UART8                             127u
                                                                /* -                  APB2 BUS                        - */
#define  BSP_PERIPH_ID_TIM1                              128u
#define  BSP_PERIPH_ID_TIM8                              129u
#define  BSP_PERIPH_ID_USART1                            132u
#define  BSP_PERIPH_ID_USART6                            133u
#define  BSP_PERIPH_ID_ADC1                              136u
#define  BSP_PERIPH_ID_ADC2                              137u
#define  BSP_PERIPH_ID_ADC3                              138u
#define  BSP_PERIPH_ID_SDIO                              139u
#define  BSP_PERIPH_ID_SPI1                              140u
#define  BSP_PERIPH_ID_SPI4                              141u
#define  BSP_PERIPH_ID_SYSCFG                            142u
#define  BSP_PERIPH_ID_TIM9                              144u
#define  BSP_PERIPH_ID_TIM10                             145u
#define  BSP_PERIPH_ID_TIM11                             146u
#define  BSP_PERIPH_ID_SPI5                              148u
#define  BSP_PERIPH_ID_SPI6                              149u
#define  BSP_PERIPH_ID_SAI1                              150u
#define  BSP_PERIPH_ID_LTDC                              154u

#endif

/*
*********************************************************************************************************
*                                           FUNCTION PROTOTYPES
*********************************************************************************************************
*/

void        BSP_Init                          (void);

void        BSP_IntDisAll                     (void);

CPU_INT32U  BSP_CPU_ClkFreq                   (void);

void        BSP_Tick_Init                     (void);

/*
*********************************************************************************************************
*                                           INTERRUPT SERVICES
*********************************************************************************************************
*/

void        BSP_IntInit                       (void);

void        BSP_IntEn                         (CPU_DATA       int_id);

void        BSP_IntDis                        (CPU_DATA       int_id);

void        BSP_IntClr                        (CPU_DATA       int_id);

void        BSP_IntVectSet                    (CPU_DATA       int_id,
                                               CPU_FNCT_VOID  isr);

void        BSP_IntPrioSet                    (CPU_DATA       int_id,
                                               CPU_INT08U     prio);



void         BOD_IRQHandler     (void);
void         IRC_IRQHandler     (void);
void         PWRWU_IRQHandler   (void);
void         RAMPE_IRQHandler   (void);
void         CLKFAIL_IRQHandler (void);
void         RTC_IRQHandler     (void);
void         TAMPER_IRQHandler  (void);
void         WDT_IRQHandler     (void);
void         WWDT_IRQHandler    (void);
void         EINT0_IRQHandler   (void);
void         EINT1_IRQHandler   (void);
void         EINT2_IRQHandler   (void);
void         EINT3_IRQHandler   (void);
void         EINT4_IRQHandler   (void);
void         EINT5_IRQHandler   (void);
void         GPA_IRQHandler     (void);
void         GPB_IRQHandler     (void);
void         GPC_IRQHandler     (void);
void         GPD_IRQHandler     (void);
void         GPE_IRQHandler     (void);
void         GPF_IRQHandler     (void);
void         SPI0_IRQHandler    (void);
void         SPI1_IRQHandler    (void);
void         BRAKE0_IRQHandler  (void);
void         PWM0P0_IRQHandler  (void);
void         PWM0P1_IRQHandler  (void);
void         PWM0P2_IRQHandler  (void);
void         BRAKE1_IRQHandler  (void);
void         PWM1P0_IRQHandler  (void);
void         PWM1P1_IRQHandler  (void);
void         PWM1P2_IRQHandler  (void);
void         TMR0_IRQHandler    (void);
void         TMR1_IRQHandler    (void);
void         TMR2_IRQHandler    (void);
void         TMR3_IRQHandler    (void);
void         UART0_IRQHandler   (void);
void         UART1_IRQHandler   (void);
void         I2C0_IRQHandler    (void);
void         I2C1_IRQHandler    (void);
void         PDMA_IRQHandler    (void);
void         DAC_IRQHandler     (void);
void         ADC00_IRQHandler   (void);
void         ADC01_IRQHandler   (void);
void         ACMP01_IRQHandler  (void);
void         ADC02_IRQHandler   (void);
void         ADC03_IRQHandler   (void);
void         UART2_IRQHandler   (void);
void         UART3_IRQHandler   (void);
void         SPI2_IRQHandler    (void);
void         USBD_IRQHandler    (void);
void         USBH_IRQHandler    (void);
void         USBOTG_IRQHandler  (void);
void         CAN0_IRQHandler    (void);
void         SC0_IRQHandler     (void);
void         TK_IRQHandler      (void);


/*
*********************************************************************************************************
*                                     PERIPHERAL POWER/CLOCK SERVICES
*********************************************************************************************************
*/

CPU_INT32U   BSP_PeriphClkFreqGet        (CPU_DATA       pwr_clk_id);

void         BSP_PeriphEn                (CPU_DATA       pwr_clk_id);

void         BSP_PeriphDis               (CPU_DATA       pwr_clk_id);



