/*
*********************************************************************************************************
*                                     MICIRUM BOARD SUPPORT PACKAGE
*
*                             (c) Copyright 2013; Micrium, Inc.; Weston, FL
*
*                   All rights reserved.  Protected by international copyright laws.
*                   Knowledge of the source code may not be used to write a similar
*                   product.  This file may only be used in accordance with a license
*                   and should not be redistributed in any way.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                     BOARD SUPPORT PACKAGE (BSP)
*
*                                       IAR Development Kits
*                                              on the
*
*                                    M451
*
* Filename      : bsp_int.c
* Version       : V1.00
* Programmer(s) : FF
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#define  BSP_INT_MODULE
#include <includes.h>

#include "M451Series.h"
/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/

#define  BSP_INT_SRC_NBR                                 64


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/

static  CPU_FNCT_VOID  BSP_IntVectTbl[BSP_INT_SRC_NBR];


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void  BSP_IntHandler     (CPU_DATA  int_id);
static  void  BSP_IntHandlerDummy(void);


/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                              BSP_IntClr()
*
* Description : Clear interrupt.
*
* Argument(s) : int_id      Interrupt to clear.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) An interrupt does not need to be cleared within the interrupt controller.
*********************************************************************************************************
*/

void  BSP_IntClr (CPU_DATA  int_id)
{

}


/*
*********************************************************************************************************
*                                              BSP_IntDis()
*
* Description : Disable interrupt.
*
* Argument(s) : int_id      Interrupt to disable.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_IntDis (CPU_DATA  int_id)
{
    if (int_id < BSP_INT_SRC_NBR) {
        CPU_IntSrcDis(int_id + 16);
    }
}


/*
*********************************************************************************************************
*                                           BSP_IntDisAll()
*
* Description : Disable ALL interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_IntDisAll (void)
{
    CPU_IntDis();
}


/*
*********************************************************************************************************
*                                               BSP_IntEn()
*
* Description : Enable interrupt.
*
* Argument(s) : int_id      Interrupt to enable.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_IntEn (CPU_DATA  int_id)
{
    if (int_id < BSP_INT_SRC_NBR) {
        CPU_IntSrcEn(int_id + 16);
    }
}


/*
*********************************************************************************************************
*                                            BSP_IntVectSet()
*
* Description : Assign ISR handler.
*
* Argument(s) : int_id      Interrupt for which vector will be set.
*
*               isr         Handler to assign
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_IntVectSet (CPU_DATA       int_id,
                      CPU_FNCT_VOID  isr)
{
    CPU_SR_ALLOC();


    if (int_id < BSP_INT_SRC_NBR) {
        CPU_CRITICAL_ENTER();
        BSP_IntVectTbl[int_id] = isr;
        CPU_CRITICAL_EXIT();
    }
}


/*
*********************************************************************************************************
*                                            BSP_IntPrioSet()
*
* Description : Assign ISR priority.
*
* Argument(s) : int_id      Interrupt for which vector will be set.
*
*               prio        Priority to assign
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_IntPrioSet (CPU_DATA    int_id,
                      CPU_INT08U  prio)
{
    CPU_SR_ALLOC();


    if (int_id < BSP_INT_SRC_NBR) {
        CPU_CRITICAL_ENTER();
        CPU_IntSrcPrioSet(int_id + 16, prio);
        CPU_CRITICAL_EXIT();
    }
}


/*
*********************************************************************************************************
*********************************************************************************************************
*                                           INTERNAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/
/*
*********************************************************************************************************
*                                              BSP_IntInit()
*
* Description : Initialize interrupts:
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : BSP_Init().
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_IntInit (void)
{
    CPU_DATA  int_id;
	

    for (int_id = 0; int_id < BSP_INT_SRC_NBR; int_id++) {
        BSP_IntVectSet(int_id, BSP_IntHandlerDummy);
    }
}


/*
*********************************************************************************************************
*                                        BSP_IntHandler####()
*
* Description : Handle an interrupt.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : This is an ISR.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BOD_IRQHandler     (void)  { BSP_IntHandler(BOD_IRQn   );                }
void  IRC_IRQHandler     (void)  { BSP_IntHandler(IRC_IRQn   );                 }
void  PWRWU_IRQHandler   (void)  { BSP_IntHandler(PWRWU_IRQn );          }
void  RAMPE_IRQHandler   (void)  { BSP_IntHandler(RAMPE_IRQn );            }
void  CLKFAIL_IRQHandler (void)  { BSP_IntHandler(CKFAIL_IRQn);               }
void  RTC_IRQHandler     (void)  { BSP_IntHandler(RTC_IRQn   );                 }
void  TAMPER_IRQHandler  (void)  { BSP_IntHandler(TAMPER_IRQn);               }
void  WDT_IRQHandler     (void)  { BSP_IntHandler(WDT_IRQn   );               }
void  WWDT_IRQHandler    (void)  { BSP_IntHandler(WWDT_IRQn  );               }
void  EINT0_IRQHandler   (void)  { BSP_IntHandler(EINT0_IRQn );               }
void  EINT1_IRQHandler   (void)  { BSP_IntHandler(EINT1_IRQn );               }
void  EINT2_IRQHandler   (void)  { BSP_IntHandler(EINT2_IRQn );            }
void  EINT3_IRQHandler   (void)  { BSP_IntHandler(EINT3_IRQn );            }
void  EINT4_IRQHandler   (void)  { BSP_IntHandler(EINT4_IRQn );            }
void  EINT5_IRQHandler   (void)  { BSP_IntHandler(EINT5_IRQn );            }
void  GPA_IRQHandler     (void)  { BSP_IntHandler(GPA_IRQn   );            }
void  GPB_IRQHandler     (void)  { BSP_IntHandler(GPB_IRQn   );            }
void  GPC_IRQHandler     (void)  { BSP_IntHandler(GPC_IRQn   );            }
void  GPD_IRQHandler     (void)  { BSP_IntHandler(GPD_IRQn   );                 }
void  GPE_IRQHandler     (void)  { BSP_IntHandler(GPE_IRQn   );             }
void  GPF_IRQHandler     (void)  { BSP_IntHandler(GPF_IRQn   );            }
void  SPI0_IRQHandler    (void)  { BSP_IntHandler(SPI0_IRQn  );            }
void  SPI1_IRQHandler    (void)  { BSP_IntHandler(SPI1_IRQn  );            }
void  BRAKE0_IRQHandler  (void)  { BSP_IntHandler(BRAKE0_IRQn);             }
void  PWM0P0_IRQHandler  (void)  { BSP_IntHandler(PWM0P0_IRQn);       }
void  PWM0P1_IRQHandler  (void)  { BSP_IntHandler(PWM0P1_IRQn);       }
void  PWM0P2_IRQHandler  (void)  { BSP_IntHandler(PWM0P2_IRQn);  }
void  BRAKE1_IRQHandler  (void)  { BSP_IntHandler(BRAKE1_IRQn);             }
void  PWM1P0_IRQHandler  (void)  { BSP_IntHandler(PWM1P0_IRQn);                }
void  PWM1P1_IRQHandler  (void)  { BSP_IntHandler(PWM1P1_IRQn);                }
void  PWM1P2_IRQHandler  (void)  { BSP_IntHandler(PWM1P2_IRQn);                }
void  TMR0_IRQHandler    (void)  { BSP_IntHandler(TMR0_IRQn  );             }
void  TMR1_IRQHandler    (void)  { BSP_IntHandler(TMR1_IRQn  );             }
void  TMR2_IRQHandler    (void)  { BSP_IntHandler(TMR2_IRQn  );             }
void  TMR3_IRQHandler    (void)  { BSP_IntHandler(TMR3_IRQn  );             }
void  UART0_IRQHandler   (void)  { BSP_IntHandler(UART0_IRQn );                }
void  UART1_IRQHandler   (void)  { BSP_IntHandler(UART1_IRQn );                }
void  I2C0_IRQHandler    (void)  { BSP_IntHandler(I2C0_IRQn  );              }
void  I2C1_IRQHandler    (void)  { BSP_IntHandler(I2C1_IRQn  );              }
void  PDMA_IRQHandler    (void)  { BSP_IntHandler(PDMA_IRQn  );              }
void  DAC_IRQHandler     (void)  { BSP_IntHandler(DAC_IRQn   );           }
void  ADC00_IRQHandler   (void)  { BSP_IntHandler(ADC00_IRQn );           }
void  ADC01_IRQHandler   (void)  { BSP_IntHandler(ADC01_IRQn );         }
void  ACMP01_IRQHandler  (void)  { BSP_IntHandler(ACMP01_IRQn);      }
void  ADC02_IRQHandler   (void)  { BSP_IntHandler(ADC02_IRQn );       }
void  ADC03_IRQHandler   (void)  { BSP_IntHandler(ADC03_IRQn );  }
void  UART2_IRQHandler   (void)  { BSP_IntHandler(UART2_IRQn );             }
void  UART3_IRQHandler   (void)  { BSP_IntHandler(UART3_IRQn );        }
void  SPI2_IRQHandler    (void)  { BSP_IntHandler(SPI2_IRQn  );                }
void  USBD_IRQHandler    (void)  { BSP_IntHandler(USBD_IRQn  );                }
void  USBH_IRQHandler    (void)  { BSP_IntHandler(USBH_IRQn  );                }
void  USBOTG_IRQHandler  (void)  { BSP_IntHandler(USBOTG_IRQn);                }
void  CAN0_IRQHandler    (void)  { BSP_IntHandler(CAN0_IRQn  );              }
void  SC0_IRQHandler     (void)  { BSP_IntHandler(SC0_IRQn   );              }
void  TK_IRQHandler      (void)  { BSP_IntHandler(TK_IRQn    );            }



/*
*********************************************************************************************************
*********************************************************************************************************
*                                           LOCAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                          BSP_IntHandler()
*
* Description : Central interrupt handler.
*
* Argument(s) : int_id          Interrupt that will be handled.
*
* Return(s)   : none.
*
* Caller(s)   : ISR handlers.
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  BSP_IntHandler (CPU_DATA  int_id)
{
    CPU_FNCT_VOID  isr;
    CPU_SR_ALLOC();


    CPU_CRITICAL_ENTER();                                       /* Tell the OS that we are starting an ISR            */

    OSIntEnter();

    CPU_CRITICAL_EXIT();

    if (int_id < BSP_INT_SRC_NBR) {
        isr = BSP_IntVectTbl[int_id];
        if (isr != (CPU_FNCT_VOID)0) {
            isr();
        }
    }

    OSIntExit();                                                /* Tell the OS that we are leaving the ISR            */
}


/*
*********************************************************************************************************
*                                        BSP_IntHandlerDummy()
*
* Description : Dummy interrupt handler.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : BSP_IntHandler().
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  BSP_IntHandlerDummy (void)
{

}
